/*
 * doctl - Do control messages that aren't implemented in relaynews.
 * rasp@trism
 *
 * Added sorting of the active file
 * alf@xenon
 *
 */


#include <stdio.h>
#include <libraries/dos.h>
#include "relay/aconf.h"

#define TRUE  1
#define FALSE 0

static char *version20 = "$VER: doctl 1.00 (5 Aug 90)\n\r";

extern char *LogProgram = "doctl";
int moderated = FALSE;

char *fgets(), *findcmd(), *strtok(), *malloc(), *strdup(), *FindConfig(), *MakeConfigPath();
void creategroup(), rmgroup(), mailadm(), sortactive();

char *activename;
char *bactivename;


main(argc, argv)
int argc;
char *argv[];
{
  activename = MakeConfigPath(UULIB, "news/active");
  bactivename = MakeConfigPath(UULIB, "news/bactive");

  if(!(strncmp(argv[1], "newgroup", 8)))
  {
    creategroup(findcmd(argv[3]));
    sortactive();
  }
  else if(!(strncmp(argv[1], "rmgroup", 6)))
  {
    rmgroup(findcmd(argv[3]));
    sortactive();
  }
  else
  {
    mailadm(argv);
  }
}

char *findcmd(name)
char *name;
{
  FILE *in;
  char *buf;
  char *cmd = 0;

  int approved = FALSE;

  if(!(buf = (char *)malloc(256)))
  {
    fprintf(stderr, "cannot allocate buffer space");
    exit(10);
  }

  if(!(in = fopen(name, "r")))
  {
    fprintf(stderr, "cannot open '%s'", name);
    exit(10);
  }

  while(fgets(buf, 256, in) && buf[0] != '\n')
  {
    if(!(strncmp(buf, "Approved:", 9)))
      approved = TRUE;

    if(!(strncmp(buf, "Control:", 8)) || !(strncmp(buf, "Subject:", 8)))
      cmd = strdup(buf);
  }

  fclose(in);

  if(approved && cmd)
    return cmd;
  else
    return NULL;
}

void creategroup(line)
char *line;
{
  FILE *active;
  char *tok,*name;
  int moderated = 0;

  if(line == NULL) 
  {
    fprintf(stderr, "warning creategroup called with null ptr");
    exit(10);
  }

  tok  = strtok(line, "\t \n");
  tok  = strtok(NULL, "\t \n");
  name = strtok(NULL, "\t \n");

  moderated = !strcmp(strtok(NULL, " \t\n"), "moderated");

  if(!(active = fopen(activename, "r")))
  {
    char buf[256];
    int found = 0;

    while((fgets(buf, 255, active)) != NULL)
    {
      if((strncmp(buf, name, strlen(name))) == 0)
      {
        found = 1;
        break;
      }
    }

    fclose(active);
    if(found != 0)
      return;
  }

  if(!(active = fopen(activename, "a")))
  {
    fprintf(stderr, "can't open '%s' for append", activename);
    exit(0);
  }

  fprintf(active, "%s 00000 00000 %c\n", name, moderated ? 'm' : 'y');
  fclose(active);
}

void rmgroup(line)
char *line;
{
  char buf[256], *name;
  FILE *active, *bactive;

  if(!(active = fopen(activename, "r")))
  {
    fprintf(stderr, "cannot open '%s'", activename);
    exit(0);
  }

  if(!(bactive = fopen(bactivename, "w")))
  {
    fprintf(stderr, "cannot open '%s'", bactivename);
    exit(10);
  }

  strtok(line, "\t \n");
  strtok(NULL, "\t \n");
  name = strtok(NULL, "\t \n");

  while(fgets(buf, 256, active))
  {
    if(strncmp(buf,name,strlen(name)))
    fputs(buf,bactive);
  }

  fclose(active);
  fclose(bactive);

  if(DeleteFile(activename) != DOSFALSE)
    Rename(bactivename, activename);
  else
    fprintf(stderr, "Can't delete '%s'", activename);
}

void mailadm(line)
char *line[];
{
  FILE *tmp,*in;
  char buf[256];
  char *uname;

  if(!(tmp = fopen("t:tmp.mail","w")))
  {
    fprintf(stderr, "Can't create t:tmp.mail","");
    exit(10);
  }

  while(*(line++))
    if(in = fopen(*line, "r"))
      break;

  if(!*line && !in)
  {
    fprintf(stderr, "Error reading control message", line);
    exit(10);
  }

  if(!(uname = FindConfig(USERNAME)))
  {
    fprintf(stderr, "Can't find UserName in config");
    exit(10);
  }

  fprintf(tmp, "From: usenet\nSubject: Control request\nTo: %s\n\n", uname);
  fprintf(tmp, "control message follows:\n");

  while(fgets(buf, 256, in))
    fputs(buf, tmp);

  fclose(in);
  fclose(tmp);

  Execute("sendmail <t:tmp.mail", 0L, 0L);
}

void sortactive()
{
  char buf[256];

  sprintf(buf, "sort from %s to %s", activename, bactivename);

  (void)Execute(buf, 0L, 0L);

  if(DeleteFile(activename) != DOSFALSE)
    Rename(bactivename, activename);
  else
    fprintf(stderr, "Cannot delete '%s'", activename);
}
