/**				fixactive.c			**/

/** Simple program to rebuild the active file to ensure that it 
    correctly points to the oldest and newest articles available
    in each of the groups on this computer.

	(C) Copyright 1990, Dave Taylor  <taylor@Intuitive.Com>
	(C) Copyright 1990, Intuitive Systems
	
	Permission is hereby given for anyone to distribute this
	software through any means, without cost to the end consumer,
	and such that the above copyright notice remains intact.
**/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "config.h"


#define MODERATED	'm'
#define UNMODERATED	'y'

#define MAXGROUPS	1000

#define NLEN		40
#define SLEN		80

#ifndef TRUE
#  define  TRUE		1
#  define  FALSE	0
#endif

#define plural(n)	( n == 1 ? "" : "s")

static char *version20 = "$VER: fixactive 1.00 (5 Aug 90)\n\r";

struct grouprec
{
  char  name[NLEN];             /* newsgroup name */
  int   oldest,                 /* oldest article */
        newest,                 /* newest article */
        moderated;              /* moderated?     */
} active[MAXGROUPS];

int    groups = 0,
       moderated = 0,
       verbose = FALSE;

char  active_file[SLEN], news_home[SLEN];
extern char *optarg;


main(argc, argv)
int argc;
char **argv;
{
	FILE *fd;
	char  buffer[SLEN], name[SLEN], modflag[3];
	int   i, j, oldest, newest, changed = FALSE, dont_fix = FALSE;
	int   bad_groups = 0, c;

	/** first pass initialization.. **/

	strcpy(active_file, MakeConfigPath(UULIB, "news/active"));
	strcpy(news_home, GetConfigDir(UUNEWS));

	/* set flags, if necessary */

	while ((c = getopt(argc, argv, "a:h:nv")) != EOF)
        {
	  switch (c)
          {
	     case 'a' : strcpy(active_file, optarg);	break;
	     case 'h' : strcpy(news_home, optarg);	break;
	     case 'n' : dont_fix = TRUE;		break;
	     case 'v' : verbose = TRUE;			break;
	     default  : usage();			exit(0);
	  }
	}

	/****  FIRST let's open the existing active file and build
	       a list of all the groups available on this computer.
	*****/

	if ((fd = fopen(active_file, "r")) == NULL)
        {
	  perror("fopen: can't open active file");
	  exit(1);
	}

	while (fgets(buffer, SLEN, fd) != NULL)
        {
	  for (j=0, i=0; buffer[i] != ' '; i++) 
	    name[j++] = buffer[i];
	  name[j] = '\0';
	  sscanf( (char *) buffer + i, "%d %d %s", 
	         &newest, &oldest, modflag);
	
	  /* and copy into our data structure */

	  strcpy(active[groups].name, name);
	  active[groups].newest = newest;
	  active[groups].oldest = oldest;
	
	  if (modflag[0] == MODERATED)
          {
	    moderated++;
	    active[groups].moderated = TRUE;
	  }
	  else 
	    active[groups].moderated = FALSE;
	
	  /* and verify... */
	
	  if (verbose)
	     printf("Group '%s', oldest = %d, newest = %d, moderated = %s\n",
		     active[groups].name, active[groups].oldest,
		     active[groups].newest, 
		     active[groups].moderated? "YES": "NO");
	
	   groups++;
	}

	(void) fclose(fd);

	if (verbose) printf("\n");

	printf("Read in %d groups total, of which %d are moderated.\n\n",
	       groups, moderated);

	/** now let's go through each group and recompute the newest
	    and oldest based on the actual information in the directory
	    for that group.
	**/

	for (i = 0; i < groups; i++)
	{
	  printf("\r%40.40s\t", active[i].name);  
	  fflush(stdout);

	  if(reset_values_for(i))
          {
	     changed = TRUE;
	     bad_groups++;
	  }
	}

	if (bad_groups)
        {
	   printf("\r                                        ");    /* clear line */
	   printf(
	      "\nFound %d group%s incorrectly marked in the active file\n\n",
	      bad_groups, plural(bad_groups));
	}
	else
	   printf("\r   .. no problems found ..                \n");

	/** And now let's rewrite the active file if there are any changes 
            that we've encountered...  **/

	if (changed)
        {
          if(dont_fix != TRUE)
          {
            rewrite_active_file();
            printf("done!\n");
          }
	}
}

int
reset_values_for(index)
int index;
{
	/** Given group[index], open that particular directory and then
	    scan through, setting lowest and highest as appropriate.
	    If there are any changes, make them in the data structure
	    and return TRUE.  Otherwise, return FALSE
	**/

	DIR *dirp;
	struct direct *dp;
	int    i, j, lowest = 9999999, highest = 0, entries_checked = 0;
	char   dirname[SLEN], directory_name[SLEN];

	/** first step is to get the right directory name... **/

	for (j=0, i=0; active[index].name[i] != '\0'; )
        {
	  if (active[index].name[i] == '.')
          {
	    dirname[j++] = '/';
	    i++;
	  }
	  else
	    dirname[j++] = active[index].name[i++];
	}
	dirname[j] = '\0';
	
	/** now prepend the home directory for netnews... **/

	sprintf(directory_name, "%s%s", news_home, dirname);
	/** and let's open this baby up! **/

	if ((dirp = opendir(directory_name)) == NULL)
        {
	  fprintf(stdout, "Does not exist? (Can't open)\n");
	  return(FALSE);
	}

	while ((dp = readdir(dirp)) != NULL)
        {
	  if (isdigit(dp->d_name[0]))
          {
	     i = atoi(dp->d_name);
	     if (i < lowest) lowest = i;
	     if (i > highest) highest = i;
	     entries_checked++;
	  }
	}

	(void) closedir(dirp);

	/** and now let's check the information out and proceed accordingly **/

	if (entries_checked == 0)	/* no articles at all! */
	  return(FALSE);

	if (active[index].oldest == lowest && active[index].newest == highest)
	  return(FALSE);

	if (active[index].oldest != lowest)
        {
	  printf("\r(reset oldest article for '%s' from %d to %d)\n",
		  active[index].name, active[index].oldest,
		  lowest);
	  active[index].oldest = lowest;
	}
	    
	if (active[index].newest != highest)
        {
	  printf("\r(reset newest article for '%s' from %d to %d)\n",
		  active[index].name, active[index].newest,
		  highest);
	  active[index].newest = highest;
	}

	return(TRUE);;
}

rewrite_active_file()
{
	/** open the active file for writing, and then replace all the
	    information therein with the information in our data structure.
	    Note the funky output format for digits:
		groupname 5-digit-integer 5-digit-integer modflag
	    where numeric values must be padded with leading zeroes,
	    and "modflag" is "m" if moderated, or "y" otherwise.
	**/

	FILE *fd;
	int   i;
	
	if ((fd = fopen(active_file, "w")) == NULL)
        {
	  fprintf(stderr, "I couldn't open file '%s' for writing!\n", active_file);
	  exit(1);
	}

	/** now line by line output the information required **/

	for (i=0 ; i < groups; i++)
	  fprintf(fd, "%s %05d %05d %c\n",
		  active[i].name, active[i].newest, active[i].oldest,
		  active[i].moderated? MODERATED : UNMODERATED);
	
	/** and we're done.  Close it and let's boogie! **/

	fclose(fd);
}

usage()
{
	/** who what when where and why and all that jazz **/

	printf("AmigaUUCP Plus: fixactive 1.00 (5 Aug 90)\n");
	printf("Written by Ingo Feulner\n\n");
	printf("Usage: fixactive [flags]\n");
	printf("Where [flags] can be any of:\n");
	
	printf("   -a fn   \tUse 'fn' as the active file\n");
	printf("           \t  (default is '%s')\n", active_file);
	printf("   -h dir  \tUse 'dir' as the netnews spool home\n");
	printf("           \t  (default is '%s')\n", news_home);
	printf("   -n      \tShow what you'd need to do, but don't do it\n");
	printf("   -v      \tVerbose: lots of output along the way\n\n");
}
