
/***************************************************************************


   MAIL.c by William Loftus
   Copyright 1988 by William Loftus.   All rights reserved.

   Changes:
           11-Jul-88 by Dan "Sneakers" Schein

                   Added .signature routine
                   Changed syntax of mail header
                   Changed beta version from 0.30 to 0.31

   Changes:
           24-Sep-88 by Dan "Sneakers" Schein

                   Removed header from user editable message
                   Check for & abort on - EMPTY message body
                   Changed beta version from 0.31 to 0.32

   Changes:
           01-Oct-88 by Dan "Sneakers" Schein

                   Fixed (added ?) processing of multiple addresses on the
                   To: line.

   Changes:
           16-Oct-88 by William Loftus

                   Re-introduced use of the uux command,
                   which replaces the function DirtyWork.
                   Added a space to GRP_DELIM.
                   Added command line arguments again.

   Changes:
           21-Nov-88 by Dan "Sneakers" Schein

                   Sent error messages to ERRORFILE, no longer printf to
                   the screen. Added the use of the config.h file and
                   removed hard coding of UUCP:[dir]/[file]
        Changes:
                        23-Dec-89 by Ralph Spitzner

                                        Added Support for reading mails
                                        Introduced use of getopt for interactive use
                                        with my Newnews Program.
        Bug:
                    03-jan-90
                                        Global array for path keeps contents if a second
                                        reply is issued.

***************************************************************************/

# include <time.h>
# include <stdio.h>
# include <libraries/dos.h>
# include <libraries/dosextens.h>
# include "config.h"

char *editor;
char *user;
char *node;
char *domain;
char to_buf[128];
char temp_to[128];
char *real_name;
char system_name[32];
char subject_buf[128];
char *temp_file_name = "T:tmp.mail";
char *temp_file_name_2 = "T:tmp.mail_head";


char *subject = 0,*incfile = 0,*username = 0, *article = 0;

FILE *ef;

char cmd[128];
char *path;
char signature[128];
char *newsfeed;
int  seq;
int  bang;
char *getmez();
void read_ctl();
void GetTo();
void GetSubject();
void GetMessage();
void TestMessage();
void BuildMail();
void Signature();
void BuildMessage();
char *FindConfig();
void do_mail();
void myexit();



#define TRUE 1
#define FALSE 0
#define GRP_DELIM " ,\n\r"
#define PROG_NAME  "Mail"


int READMODE = 0;
int QUIETMODE = 0;


int
is_in_L_sys_file(system_name)
char *system_name;
{
  char buf[256];
  char system[128];
  FILE *fd;

  if (!(fd = fopen("UUCP:lib/L.sys","r"))) {
    fprintf(ef, "%s: Couldn't open UUCP:lib/L.sys file\n", PROG_NAME);
    chdir(path);
        fclose(ef);
    myexit(1);
  }

  sprintf(system,"%s ", system_name);

  while (NULL != fgets(buf, sizeof buf, fd)) {
    if (buf[0] == '#') continue;
    if (strncmp(buf, system, strlen(system)) == 0) {
      fclose(fd);
      return TRUE;
    }
  }

  fclose(fd);
  return FALSE;

}


void
main(argc, argv)
  int argc;
  char **argv;
{
  extern char *optarg;
  extern int optind;
  char c;
  if (!(ef = fopen(ERRORFILE,"a")))
        {
    printf("Can not open ERROR file %s\n", ERRORFILE);
        myexit(1);
        }

  path = (char *)getcwd(path);
  if(!(editor = FindConfig("MailEditor")))
                myexit(printf("MailEditor entry in configfile is missing\n"));
  chdir("uucp:spool");
  read_ctl();
  strcpy(&to_buf,newsfeed);

        while((c = getopt(argc,argv,"u:s:f:a:q")) > 0)
                {
                switch(c)
                        {
                        case 'u':username = optarg;
                                     break;
                        case 's':subject  = optarg;
                                         break;
                        case 'f':incfile  = optarg;
                                         break;
                        case 'a':article = optarg;
                                         break;
                        case 'q': QUIETMODE = TRUE;
                                         break;
                        }
                }

strcpy(&to_buf,newsfeed);
strcat(&to_buf,"!");
do_mail();
chdir(path);
fclose(ef);
myexit(0);
}


void
do_mail()
{

  to_buf[strlen(newsfeed) + 1 ] = 0;
  if(username == 0)
                GetTo();
  else  {
                strcat(&to_buf,username);
                strcat(&to_buf,"\n");
                }

  if(subject == 0)
                GetSubject();
  else strcpy(subject_buf,subject);


  printf("%s\n",incfile);
  if( incfile && QUIETMODE)
        {
        IncludeQuiet(incfile);
        }
  else
        if(incfile)
                Include(incfile);
  GetMessage();

  TestMessage();
  BuildMessage();
  BuildMail();
 }

void
GetTo()
{
   printf("To: ");
   fgets(&to_buf[strlen(&to_buf)], 80, stdin);
}

void
GetSubject()
{
   printf("Subject: ");
   fgets(subject_buf, 128, stdin);
}

void
GetMessage()
{
  FILE *fp;
  FILE *mh;
  char buf[128];
  long t;

  fp = fopen("uucp:lib/seq","r");
  if (fp)
        {
    fgets(buf, 128, fp);
    seq = atoi(buf);
    fclose(fp);
        }
else
        {
    fprintf(ef, "%s: Can't open UUCP:lib/seq for reading.\n", PROG_NAME);
    perror("UUCP:lib/seq");
    chdir(path);
        fclose(ef);
    myexit(1);
        }


  fp = fopen("uucp:lib/seq", "w");

  if (fp)
        {
    fprintf(fp,"%d", seq+4);
    fclose(fp);
        }
 else
        {
    fprintf(ef, "%s: Can't open UUCP:lib/seq for writing.\n", PROG_NAME);
    perror("UUCP:lib/seq");
    chdir(path);
        fclose(ef);
    myexit(1);
        }

  mh = fopen(temp_file_name_2, "w");

  if (mh)
        {

    time(&t);
    fprintf(mh,"Date: %s\n", getmez());
    fprintf(mh,"From: %s@%s%s\n", user, node, domain);
    fprintf(mh,"To: %s", to_buf+strlen(newsfeed) +1 );
    fprintf(mh,"Subject: %s", subject_buf);

    fclose(mh);

        }
else
        {
    fprintf(ef, "%s: Can't open temp file for %s\n",PROG_NAME,temp_file_name_2);
    perror(temp_file_name_2);
    chdir(path);
        fclose(ef);
    myexit(1);
        }

  fp = fopen(temp_file_name, "a");

  if (fp)
        {
    fclose(fp);
        }
else
        {
    fprintf(ef, "%s: Can't open temp file for %s\n",PROG_NAME,temp_file_name);
    perror(temp_file_name);
    chdir(path);
        fclose(ef);
    myexit(1);
        }

  chdir(path);
  fexecl(editor,editor,temp_file_name,NULL);
for(;;)
	{
  	printf("[S]end [E]dit [H]eader [A]bort ?");
	switch((getchar()|0x20))
		{
		case 's': goto send; /* how would YOU break the loop ? */
		case 'a': myexit();
		case 'e': fexecl(editor,editor,temp_file_name,NULL);
				  break;
		case 'h': fexecl(editor,editor,temp_file_name_2,NULL);
				  break;
		}
	fflush(stdin);
	}
send:
  fflush(stdin);
  printf("Append .signature file? [n/y] ");
  fgets(signature, 3, stdin);
  if ((signature[0] | 0x20) == 'y')
  {
     Signature();
     printf(".signature file appended.\n");
  }

  chdir("UUCP:spool");

}

void
BuildMail()
{
   strcpy (temp_to, strtok(to_buf, GRP_DELIM));
   sprintf(cmd, "sendmail <%s ",temp_file_name);
   Execute(cmd,0,0);

   DeleteFile(temp_file_name);
}


/*
 * Read the control file and grab a few parameters.
 */
void
read_ctl()
{

if(!(user = FindConfig("UserName")))
        myexit(printf("cannot find username\n"));
if(!(node = FindConfig("NodeName")))
        myexit(printf("cannot find NodeName\n"));
if(!(domain = FindConfig("DomainName")))
	myexit(printf("cannot find DomainName"));
if(!(real_name = FindConfig("RealName")))
        myexit(printf("cannot find RealName\n"));
if(!(newsfeed = FindConfig("NewsFeed")))
        myexit(printf("cannot find NewsFeed\n"));
}

void
Signature()
{
   FILE *fp;
   FILE *sf;
   char buff[128];

   fp = fopen(temp_file_name, "a");
   sf = fopen("uucp:lib/.signature", "r");

   if (sf)
   {
	  fprintf(fp,"\n--\n");
      while (NULL != fgets(buff, sizeof buff, sf))
      {
         fprintf(fp, "%s", buff);
      }
   fflush(fp);
   fclose(sf);
   }
   else
   {
      fprintf(ef, "%s: Can't open UUCP:lib/.signature file\n", PROG_NAME);
   }

   fclose(fp);
}

void
BuildMessage()
{
   FILE *fp;
   FILE *mh;
   char buff[128];

   fp = fopen(temp_file_name, "r");
   mh = fopen(temp_file_name_2, "a");
   fprintf(mh,"\n");
   while (NULL != fgets(buff, sizeof buff, fp))
      fprintf(mh, "%s", buff);

   fclose(fp);
   fclose(mh);

   DeleteFile(temp_file_name);
   rename(temp_file_name_2, temp_file_name);
}

void
TestMessage()
{
   FILE *fp;
   char buff[128];

   fp = fopen(temp_file_name, "r");
   fgets(buff, sizeof buff, fp);
   fclose(fp);
   if (strlen(buff) <= 0) {
      printf("No Message - Message aborted\n");
      perror("Empty Message");
      chdir(path);
          fclose(ef);
      myexit(3);
   }
}


Include(filename)
char *filename;
{
FILE *inc,*fp;
char buffer[128];

if(!(inc = fopen(filename,"r")))
        return(0);
if(!(fp = fopen(temp_file_name,"w")))
        {
        fclose(inc);
        return(0);
        }
if(username)
        fprintf(fp,"%s writes:\n",username);
while(fgets(buffer,128,inc))
        fprintf(fp,"> %s",buffer);
fclose(fp);
fclose(inc);
}

IncludeQuiet(name)
char *name;
{
FILE *in,*out;
char buf[128];
if(!(in = fopen(name,"r")))
        return(0);

if(!(out = fopen(temp_file_name,"w")))
        myexit(printf("Cannot open tempfile\n"));
while(fgets(buf,128,in))
        if(buf[0] == '\n') break;
fprintf(out,"In article (%s) you write:\n",article);
while(fgets(buf,128,in))
        fprintf(out,"> %s",buf);
fclose(in);
fclose(out);
return(0);
}


void
myexit(code)
int code;
{
chdir(path);
exit(code);
}
