/*
 *  newscontrol.c
 *
 *  - execute USENET control messages.
 *
 *  Copyright (C) 1990 Ingo Feulner, Magic Soft Developments.
 *  - all rights reserved.
 *
 *  17 Apr 91: Corrected a bug in do_rmgroup.
 */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <proto/dos.h>
#include "//include/config.h"

#define FAIL    (-1)
#define SUCCESS (0)

#define MAXGROUPLEN 128
#define SEP "\n\t "

#define TEMPFILE "T:newscontrol.tmp"


static char *version20 = "$VER: newscontrol 0.91 (17 Apr 91)\n\r";

struct activentry
{
  struct activentry *next;
  char group[MAXGROUPLEN];
  long first;
  long last;
  char ismod;
};

/* Prototypes */

/* extern */
void *lsort(void *, int, int(*)());
char *MakeConfigPath();

/* local */
int mycmp(struct activentry *, struct activentry *);
struct activentry *     readin_active(void);
int                     write_active(struct activentry *);
char *                  check_approved(char *);
char *                  get_address(char *);
void                    do_newgroup(struct activentry **, char *);
void                    do_rmgroup(struct activentry **, char *);
void                    do_sendsys(char *);
void                    do_uuname(char *);
void                    do_version(char *);
void                    send_mail(void);
void *__regargs         xmalloc(int);

char *NodeName;
char *DomainName;

short SendControl = 0;

int mycmp(struct activentry *a, struct activentry *b)
{
  return strcmp(a->group, b->group);
}

void *__regargs xmalloc(int size)
{
  void *ptr;

  if((ptr = malloc(size)) != NULL)
    return ptr;
  else
  {
    fprintf(stderr, "Insufficient free store.\n");
    exit(20);
  }
}

/*
   First Arg : control-command
   Second Arg: path of article relative to uunews:
*/
void main(int argc, char **argv)
{
  struct activentry *active;
  char *command;
  char *ControlMsgs;

  NodeName    = FindConfig("NodeName");
  DomainName  = FindConfig("DomainName");

  ControlMsgs = FindConfig("ControlMsgs");
  if((strcmp(ControlMsgs, "Yes")) == 0)
    SendControl = 1;


  if((strcmp(argv[1], "newgroup")) == 0)
  {
    active = readin_active();

    command = check_approved(argv[2]);

    if(command != NULL)
    {
      do_newgroup(&active, command);
      (void)write_active(active);
    }
  }
  else if((strcmp(argv[1], "rmgroup")) == 0)
  {
    active = readin_active();

    command = check_approved(argv[2]);

    if(command != NULL)
    {
      do_rmgroup(&active, command);
      (void)write_active(active);
    }
  }
  else if((strcmp(argv[1], "sendsys")) == 0)
  {
    do_sendsys(get_address(argv[2]));
  }
  else if((strcmp(argv[1], "uuname")) == 0)
  {
    do_uuname(get_address(argv[2]));
  }
  else if((strcmp(argv[1], "version")) == 0)
  {
    do_version(get_address(argv[2]));
  }
  else
  {
    
  }
}

void do_rmgroup(struct activentry **active, char *line)
{
  struct activentry *q, *p, *z;

  char *command;
  char *newsgroup;

  command   = strtok(line, SEP);
  if((strcmp(command, "rmgroup")) != 0)
    return;

  newsgroup = strtok(NULL, SEP);


  for(q = NULL, p = *active; p != NULL; )
  {
    if((strcmp(p->group, newsgroup)) == 0)
    {
      if(q == NULL) /* Special case: Delete first group */
      {
        z = *active;
        *active = z->next;
      }
      else
      {
        z = q->next;
        q->next = p->next;
      }
      free(z);
    }

    q = p;
    p = q->next;
  }
}

void do_newgroup(struct activentry **active, char *line)
{
  struct activentry *p, *q;

  char *command;
  char *newsgroup;
  int moderated;

  command = strtok(line, SEP);
  newsgroup = strtok(NULL, SEP);
  moderated = (*(strtok(NULL, SEP)) == 'm') ? SUCCESS : FAIL;

  if((strcmp(command, "newgroup")) != 0)
    return;

  for(p = *active; p != NULL; ) /* search for end of list */
  {
    if((strcmp(p->group, newsgroup)) == 0) /* Newsgroup does already exist */
      return;

    q = p;
    p = p->next;
  }

  p = xmalloc(sizeof(struct activentry));

  strcpy(p->group, newsgroup);
  p->first = 0;
  p->last  = 0;
  p->ismod = (moderated != FAIL) ? 'm' : 'y';
  p->next  = NULL;

  q->next = p; /* Add new entry */

  p = lsort(*active, 0, mycmp); /* and sort the list again */
  *active = p;
}

void do_sendsys(char *address)
{
  FILE *fp;

  if(address == NULL)
  {
    fprintf(stderr, "sendsys() called with NULL-address");
    exit(20);
  }

  if((fp = fopen(TEMPFILE, "w")) != NULL)
  {
    FILE *sysfile;

    fprintf(fp, "From: news@%s%s (News Administration)\n", NodeName, DomainName);

    if(SendControl != 0) /* Sending sys file is allowed */
    {
      fprintf(fp, "To: %s", address);
      fprintf(fp, "Cc: postmaster@%s%s\n", NodeName, DomainName);

      fprintf(fp, "Subject: Request for sys-file from site %s%s\n\n",
                   NodeName, DomainName);

      fprintf(fp, "Dear Sender!\n"\
                  "You requested our 'sys'-file, it is appended to this message\n"\
                  "This 'sys'-file is currently running on %s%s.\n\n", 
                  NodeName, DomainName);
      fprintf(fp, "** NOTE: This 'sys'-format isn't compatible to this of BNews or CNews,\n"\
                  "         so if you have got some problems, be free and send a mail\n"\
                  "         to postmaster@%s%s\n\n", NodeName, DomainName);
      fprintf(fp, "The path to here is: 'Path: %s!news'.\n\n\n", NodeName);

      if((sysfile = fopen(MakeConfigPath(UULIB, "news/sys"), "r")) != NULL)
      {
        char *buf = xmalloc(1024);

        while((fgets(buf, 1023, sysfile)) != NULL)
          fputs(buf, fp);

        free(buf);
        fclose(sysfile);
      }
    }
    else
    {
      fprintf(fp, "To: postmaster\n"\
                  "Subject: sendsys control message\n\n");

      fprintf(fp, "User %s sent a 'sendsys' control message\n", address);
      fprintf(fp, "Sending our 'sys'-file is disabled by the config entry 'ControlMsgs'\n");
      fprintf(fp, "\n\n\tGreets, your news administration.\n");
    }
    fclose(fp);

    send_mail();
  }

  (void)remove(TEMPFILE);
}

void do_version(char *address)
{
  FILE *fp;

  if(address == NULL)
  {
    fprintf(stderr, "version() called with NULL-address");
    exit(20);
  }

  if((fp = fopen(TEMPFILE, "w")) != NULL)
  {
    fprintf(fp, "From: news@%s%s (News Administration)\n", NodeName, DomainName);

    if(SendControl != 0) /* Sending sys file is allowed */
    {
      fprintf(fp, "To: %s", address);
      fprintf(fp, "Cc: postmaster@%s%s\n", NodeName, DomainName);

      fprintf(fp, "Subject: Our news version\n\n");

      fprintf(fp, "Currently running AmigaUUCP Plus 1.00\n");
    }
    else
    {
      fprintf(fp, "To: postmaster\n"\
                  "Subject: sendsys control message\n\n");

      fprintf(fp, "User %s sent a 'version' control message\n", address);
      fprintf(fp, "Sending our news version is disabled by the config entry 'ControlMsgs'\n");
      fprintf(fp, "\n\n\tGreets, your news administration.\n");
    }
    fclose(fp);

    send_mail();
  }
  (void)remove(TEMPFILE);
}

void do_uuname(char *address)
{
}

void send_mail(void)
{
  char buf[256];

  sprintf(buf, "sendmail <%s", TEMPFILE);

  (void)Execute(buf, 0L, 0L);
}

char *get_address(char *path)
{
  FILE *fp;
  char *from = NULL;
  char *replyto = NULL;

  if((fp = fopen(path, "r")) != NULL)
  {
    char buf[256];

    while((fgets(buf, sizeof(buf), fp)) != NULL)
    {
      if(buf[0] == '\n') /* EOH */
        break;

      if((strncmp(buf, "Reply-To: ", 10)) == 0)
      {
        replyto = xmalloc(strlen(buf));
        strcpy(replyto, &buf[10]);
      }
      else if((strncmp(buf, "From: ", 6)) == 0)
      {
        from = xmalloc(strlen(buf));
        strcpy(from, &buf[6]);
      }
    }
    fclose(fp);
  }

  if(replyto != NULL && *replyto != '\0')
    return replyto;
  else
    return from;
}

char *check_approved(char *name)
{
  FILE *fp;
  char *path;
  char *command = NULL;

  int flag = FAIL;

/*  path = MakeConfigPath(UUNEWS, name); */
  path = name;

  if((fp = fopen(path, "r")) != NULL)
  {
    char buf[256];

    while((fgets(buf, 255, fp)) != NULL)
    {
      if(buf[0] == '\n') /* End of header */
        break;

      if((strncmp(buf, "Approved: ", 10)) == 0)
      {
        flag = SUCCESS; /* Yeah, this article is approved */
      }

      if(((strncmp(buf, "Control: ", 9)) == 0)
        || ((strncmp(buf, "Subject: ", 9)) == 0))
      {
        if(command != NULL)
          free(command);

        command = xmalloc(strlen(buf));
        strcpy(command, &buf[9]);
      }
    }
    fclose(fp);
  }

  if(flag == SUCCESS && command != NULL)
    return command;
  else
    return NULL;
}
  
struct activentry *readin_active(void)
{
  FILE *act;

  struct activentry active;
  struct activentry *q, *p;

  active.next = NULL;

  act = fopen(MakeConfigPath(UULIB, "news/active"), "r");

  if(act != NULL)
  {
    char *buf = xmalloc(512);
    char tmp[10];

    q = &active;

    while((fgets(buf, 511, act)) != NULL)
    {
      p = xmalloc(sizeof(struct activentry));

      sscanf(buf, "%s %ld %ld %s\n", p->group, &p->last, &p->first, tmp);
      p->ismod = tmp[0];

      q->next = p;
      p->next = NULL;
      q = p;
    }
    fclose(act);

    p = lsort(active.next, 0, mycmp); /* Sort the linked list */
    return p;
  }
  else
    return NULL;
}

int write_active(struct activentry *active)
{
  FILE *act;
  struct activentry *p;

  char *oldactive = MakeConfigPath(UULIB, "news/active.o");
  char *newactive = MakeConfigPath(UULIB, "news/active");

  (void)remove(oldactive);              /* remove old backed-up active */
  (void)rename(newactive, oldactive);   /* and rename current to old   */

  if((act = fopen(newactive, "w")) == NULL)
    return FAIL;

  for(p = active; p != NULL; p = p->next)
    fprintf(act, "%s %05ld %05ld %c\n", p->group, p->last, p->first, p->ismod);

  if(ferror(act))
    return FAIL;

  fclose(act);

  free(oldactive);
  free(newactive);

  return SUCCESS;
}
