
/*
 * RNEWS.C
 *
 * Copyright 1988 by William Loftus.  All rights reserved. (C) Copyright
 * 1989-1990 by Matthew Dillon,  All Rights Reserved.
 *
 * Version 0.60 Beta
 *
 * This is rnews for compressed news.   News 2.11 will uux a file to this
 * system that will be in compressed format. This program will strip off the
 * "#! cunbatch", uncompress the news, and call unbatch.  If the news is not
 * in compressed format it will just pass it to unbatch.
 */

/*
 *  Changes for AmigaUUCP Plus 1.00 Copyright (C) 1990 by Ingo Feulner
 *
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <ctype.h>
#include "log.h"


static char *version20 = "$VER: rnews 1.00 (5 Aug 90)\n\r";

FILE *popen(char *, char *);
int  pclose(FILE *);


void            uncompress_news();
void            writeerr();
char           *TmpFileName();

int             Compressed = 0;
char           *TmpFile;

FILE *pout; /* File pointer for pipe */

void main(argc, argv)
int             argc;
char          **argv;
{
    FILE           *fp;
    char            buf[32];
    int             ch;
    int             status = 0;


    LogProgram = "RNews";

    fgets(buf, 13, stdin);

    if (!strncmp(buf, "#! cunbatch", 11))
    {
        Compressed = 1;
    }
    else if (!strncmp(buf, "#! rnews", 8))
    {
        fp = popen("relaynews -r", "w"); /* Create child process */
        if (fp == NULL)
        {
            ulog(-1, "Unable to fork %s", "relaynews");
            exit(1);
        }

        fprintf(fp, "%s", buf);

        while ((ch = getchar()) != EOF)
        {
            fputc(ch, fp);
        }

        status = pclose(fp); /* Close child process */
        if(status != 0)
          ulog(-1, "relaynews failed");

        Compressed = 0;
    }
    else
    {
        ulog(-1, "News in wrong format");
        remove(TmpFile);
        exit(1);
    }

    if (Compressed)
    {
        uncompress_news();
    }
}

/*
 * Set USERMEM to the maximum amount of physical user memory available in
 * bytes.  USERMEM is used to determine the maximum BITS that can be used for
 * compression.
 *
 * SACREDMEM is the amount of physical memory saved for others; compress will
 * hog the rest.
 */
#define PBITS 16

#ifdef PBITS                    /* Preferred BITS for this memory size */
#ifndef BITS
#define BITS PBITS
#endif  /* BITS */
#endif                          /* PBITS */

#if BITS == 16
#define HSIZE   69001           /* 95% occupancy */
#endif
#if BITS == 15
#define HSIZE   35023           /* 94% occupancy */
#endif
#if BITS == 14
#define HSIZE   18013           /* 91% occupancy */
#endif
#if BITS == 13
#define HSIZE   9001            /* 91% occupancy */
#endif
#if BITS <= 12
#define HSIZE   5003            /* 80% occupancy */
#endif

#ifdef M_XENIX                  /* Stupid compiler can't handle arrays with */
#if BITS == 16                  /* more than 65535 bytes - so we fake it */
#define XENIX_16
#else
#if BITS > 13                   /* Code only handles BITS = 12, 13, or 16 */
#define BITS 13
#endif
#endif
#endif

/*
 * a code_int must be able to hold 2**BITS values of type int, and also -1
 */
#if BITS > 15
typedef long int code_int;

#else
typedef int     code_int;

#endif

#ifdef SIGNED_COMPARE_SLOW
typedef unsigned long int count_int;
typedef unsigned short int count_short;

#else
typedef long int count_int;

#endif

#ifdef NO_UCHAR
typedef char    char_type;

#else
typedef unsigned char char_type;

#endif                          /* UCHAR */
char_type       magic_header[] =
{"\037\235"};                   /* 1F 9D */

/* Defines for third byte of header */
#define BIT_MASK        0x1f
#define BLOCK_MASK      0x80
/*
 * Masks 0x40 and 0x20 are free.  I think 0x20 should mean that there is a
 * fourth header byte (for expansion).
 */
#define INIT_BITS 9             /* initial number of bits/code */

#include <stdio.h>
#include <ctype.h>

int             n_bits;         /* number of bits/code */
int             maxbits = BITS; /* user settable max # bits/code */
code_int        maxcode;        /* maximum code, given n_bits */
code_int        maxmaxcode = 1 << BITS; /* should NEVER generate this code */

#define MAXCODE(n_bits)        ((1 << (n_bits)) - 1)

count_int       *htab;
unsigned short  *codetab;

#define htabof(i)       htab[i]
#define codetabof(i)    codetab[i]
code_int        hsize = HSIZE;  /* for dynamic table sizing */
count_int       fsize;

/*
 * To save much memory, we overlay the table used by compress() with those
 * used by decompress().  The tab_prefix table is the same size and type as
 * the codetab.  The tab_suffix table needs 2**BITS characters.  We get this
 * from the beginning of htab.  The output stack uses the rest of htab, and
 * contains characters.  There is plenty of room for any possible stack
 * (stack used to be 8000 characters).
 */

#define tab_prefixof(i) codetabof(i)
#define tab_suffixof(i)        (((char_type *)htab)[i])
#define tab_suffixof1(i)        (((long)htab)+(long)(i))
#define de_stack               (char_type *)(tab_suffixof1(1<<BITS))

code_int        free_ent = 0;   /* first unused entry */
int             exit_stat = 0;

code_int        getcode();

int             nomagic = 0;    /* Use a 3-byte magic number header, unless
                                 * old file */

/*
 * block compression parameters -- after all codes are used up, and
 * compression rate changes, start over.
 */
int             block_compress = BLOCK_MASK;
int             clear_flg = 0;
long int        ratio = 0;

#define CHECK_GAP 10000         /* ratio check interval */
count_int       checkpoint = CHECK_GAP;

/*
 * the next two codes should not be changed lightly, as they must not lie
 * within the contiguous general code space.
 */
#define FIRST   257             /* first free entry */
#define CLEAR   256             /* table clear output code */

int             (*bgnd_flag) ();

static void     decompress();

void
uncompress_news()
{
    pout = popen("relaynews -r", "w"); /* Create Child process */
    if(pout == NULL)
    {
        ulog(-1, "Can't fork relaynews");
        exit(1);
    }

    maxbits = BITS;
    maxmaxcode = 1 << maxbits;

    /* Check the magic number */
    if (nomagic == 0)
    {
        if ((getchar() != (magic_header[0] & 0xFF))
            || (getchar() != (magic_header[1] & 0xFF)))
        {
            ulog(-1, "stdin: not in compressed format");
            exit(1);
        }
        maxbits = getchar();    /* set -b from file */
        block_compress = maxbits & BLOCK_MASK;
        maxbits &= BIT_MASK;
        maxmaxcode = 1 << maxbits;
        fsize = 100000;         /* assume stdin large for USERMEM */
    }
    decompress();

    if((pclose(pout)) != 0) /* Close child process */
      ulog(-1, "relaynews failed");
}

static int      offset;
long int        in_count = 1;   /* length of input */
long int        bytes_out;      /* length of compressed output */
long int        out_count = 0;  /* # of codes output (for debugging) */


/*
 * Decompress stdin to stdout.  This routine adapts to the codes in the file
 * building the "string" table on-the-fly; requiring no table to be stored in
 * the compressed file.  The tables used herein are shared with those of the
 * compress() routine.  See the definitions above.
 */

static void
decompress()
{
    register char_type *stackp;
    register int    finchar;
    register code_int code, oldcode, incode;

/*---------------------------------------------------------*/

        if(!(codetab = (unsigned short *)malloc(sizeof(unsigned short) * HSIZE)))
                {
                fprintf(stderr, "compress: out of mem\n");
                }
        if(!(htab = (count_int *)malloc(sizeof(count_int) * HSIZE)))
                {
                fprintf(stderr, "compress: out of mem\n");
                }
/*---------------------------------------------------------*/



    /*
     * As above, initialize the first 256 entries in the table.
     */
    maxcode = MAXCODE(n_bits = INIT_BITS);
    for (code = 255; code >= 0; code--)
    {
        tab_prefixof(code) = 0;
        tab_suffixof(code) = (char_type) code;
    }
    free_ent = ((block_compress) ? FIRST : 256);

    finchar = oldcode = getcode();
    if (oldcode == -1)          /* EOF already? */
        return;                 /* Get out of here */
    fputc((char) finchar, pout);    /* first code must be 8 bits = char */
    if (ferror(pout))         /* Crash if can't write */
        writeerr();
    stackp = de_stack;

    while ((code = getcode()) > -1)
    {

        if ((code == CLEAR) && block_compress)
        {
            for (code = 255; code >= 0; code--)
                tab_prefixof(code) = 0;
            clear_flg = 1;
            free_ent = FIRST - 1;
            if ((code = getcode()) == -1)       /* O, untimely death! */
                break;
        }
        incode = code;
        /*
         * Special case for KwKwK string.
         */
        if (code >= free_ent)
        {
            *stackp++ = finchar;
            code = oldcode;
        }
        /*
         * Generate output characters in reverse order
         */
#ifdef SIGNED_COMPARE_SLOW
        while (((unsigned long) code) >= ((unsigned long) 256))
        {
#else
        while (code >= 256)
        {
#endif
            *stackp++ = tab_suffixof(code);
            code = tab_prefixof(code);
        }
        *stackp++ = finchar = tab_suffixof(code);

        /*
         * And put them out in forward order
         */
        do
            fputc(*--stackp, pout);
        while (stackp > de_stack);

        /*
         * Generate the new entry.
         */
        if ((code = free_ent) < maxmaxcode)
        {
            tab_prefixof(code) = (unsigned short) oldcode;
            tab_suffixof(code) = finchar;
            free_ent = code + 1;
        }
        /*
         * Remember previous code.
         */
        oldcode = incode;
    }

    free(codetab);
    free(htab);

    fflush(pout);
    if(ferror(pout))
        writeerr();
}

char_type       lmask[9] =
{0xff, 0xfe, 0xfc, 0xf8, 0xf0, 0xe0, 0xc0, 0x80, 0x00};
char_type       rmask[9] =
{0x00, 0x01, 0x03, 0x07, 0x0f, 0x1f, 0x3f, 0x7f, 0xff};

/*****************************************************************
 * TAG( getcode )
 *
 * Read one code from the standard input.  If EOF, return -1.
 * Inputs:
 *      stdin
 * Outputs:
 *      code or -1 is returned.
 */

code_int
getcode()
{
    /*
     * On the VAX, it is important to have the register declarations in
     * exactly the order given, or the asm will break.
     */
    register code_int code;
    static int      offset = 0, size = 0;
    static char_type buf[BITS];
    register int    r_off, bits;
    register char_type *bp = buf;

    if (clear_flg > 0 || offset >= size || free_ent > maxcode)
    {
        /*
         * If the next entry will be too big for the current code size, then
         * we must increase the size.  This implies reading a new buffer
         * full, too.
         */
        if (free_ent > maxcode)
        {
            n_bits++;
            if (n_bits == maxbits)
                maxcode = maxmaxcode;   /* won't get any bigger now */
            else
                maxcode = MAXCODE(n_bits);
        }
        if (clear_flg > 0)
        {
            maxcode = MAXCODE(n_bits = INIT_BITS);
            clear_flg = 0;
        }
        size = fread(buf, 1, n_bits, stdin);
        if (size <= 0)
            return -1;          /* end of file */
        offset = 0;
        /* Round size down to integral number of codes */
        size = (size << 3) - (n_bits - 1);
    }
    r_off = offset;
    bits = n_bits;
    /*
     * Get to the first byte.
     */
    bp += (r_off >> 3);
    r_off &= 7;
    /* Get first part (low order bits) */
#ifdef NO_UCHAR
    code = ((*bp++ >> r_off) & rmask[8 - r_off]) & 0xff;
#else
    code = (*bp++ >> r_off);
#endif                          /* NO_UCHAR */
    bits -= (8 - r_off);
    r_off = 8 - r_off;          /* now, offset into code word */
    /* Get any 8 bit parts in the middle (<=1 for up to 16 bits). */
    if (bits >= 8)
    {
#ifdef NO_UCHAR
        code |= (*bp++ & 0xff) << r_off;
#else
        code |= *bp++ << r_off;
#endif                          /* NO_UCHAR */
        r_off += 8;
        bits -= 8;
    }
    /* high order bits. */
    code |= (*bp & rmask[bits]) << r_off;
    offset += n_bits;

    return code;
}

char *rindex(s, c)                  /* For those who don't have it in libc.a */
char           *s, c;
{
    char           *p;

    for (p = NULL; *s; s++)
        if (*s == c)
            p = s;
    return (p);
}

void
writeerr()
{
    perror(TmpFile);
    unlink(TmpFile);
    exit(1);
}
